@@ -16,11 +16,12 @@ require 'zpng/scan_line'
 require 'zpng/scan_line/mixins'
 require 'zpng/chunk'
 require 'zpng/text_chunk'
-require 'zpng/readable_struct'
 require 'zpng/adam7_decoder'
 require 'zpng/hexdump'
 require 'zpng/metadata'
 require 'zpng/pixels'
 
 require 'zpng/bmp/reader'
+require 'zpng/jpeg/chunks'
+require 'zpng/jpeg/reader'
 require 'zpng/image'
