@@ -1,8 +1,9 @@
+# -*- coding:binary; frozen_string_literal: true -*-
 require 'stringio'
 
 module ZPNG
   class Image
-    attr_accessor :chunks, :scanlines, :imagedata, :extradata, :format, :verbose
+    attr_accessor :chunks, :scanlines, :extradata, :format, :verbose
 
     # now only for (limited) BMP support
     attr_accessor :color_class
@@ -12,15 +13,21 @@ module ZPNG
     alias :dup   :deep_copy
 
     include BMP::Reader
+    include JPEG::Reader
 
-    PNG_HDR = "\x89PNG\x0d\x0a\x1a\x0a".force_encoding('binary')
-    BMP_HDR = "BM".force_encoding('binary')
+    PNG_HDR = "\x89PNG\x0d\x0a\x1a\x0a"
 
     # possible input params:
     #   IO      of opened image file
     #   String  with image file already readed
     #   Hash    of image parameters to create new blank image
+    #   width, height
     def initialize x, h={}
+      if x.is_a?(Numeric) && h.is_a?(Numeric)
+        x = { width: x, height: h }
+        h = {}
+      end
+
       @chunks = []
       @extradata = []
       @color_class = Color
@@ -74,6 +81,18 @@ module ZPNG
       end
       alias :load_file :load
       alias :from_file :load # as in ChunkyPNG
+
+      def from_rgb data, width:, height:
+        img = new(width: width, height: height, bpp: 24)
+        img.scanlines = height.times.map{ |i| ScanLine.new(img, i, decoded_bytes: data[width*3*i, width*3]) }
+        img
+      end
+
+      def from_rgba data, width:, height:
+        img = new(width: width, height: height, bpp: 32)
+        img.scanlines = height.times.map{ |i| ScanLine.new(img, i, decoded_bytes: data[width*4*i, width*4]) }
+        img
+      end
     end
 
     # save image to file
@@ -154,11 +173,13 @@ module ZPNG
       # Once a stream is in binary mode, it cannot be reset to nonbinary mode.
       io.binmode
 
-      hdr = io.read(BMP_HDR.size)
-      if hdr == BMP_HDR
+      hdr = io.read(BMP::MAGIC.size)
+      if hdr == BMP::MAGIC
         _read_bmp io
+      elsif hdr == JPEG::MAGIC
+        _read_jpeg io
       else
-        hdr << io.read(PNG_HDR.size - BMP_HDR.size)
+        hdr << io.read(PNG_HDR.size - BMP::MAGIC.size)
         if hdr == PNG_HDR
           _read_png io
         else
@@ -276,7 +297,7 @@ module ZPNG
     # on errors keep going and try to return maximum possible data
     def _safe_inflate data
       zi = Zlib::Inflate.new
-      pos = 0; r = ''
+      pos = 0; r = String.new
       begin
         # save some memory by not using String#[] when not necessary
         r << zi.inflate(pos==0 ? data : data[pos..-1])
@@ -326,6 +347,11 @@ module ZPNG
         end
     end
 
+    def imagedata= data
+      @scanlines = nil
+      @imagedata = data
+    end
+
     def imagedata_size
       if new_image?
         @scanlines.map(&:size).inject(&:+)
@@ -409,6 +435,18 @@ module ZPNG
       end
     end
 
+    def to_ansi wide: false
+      spc = wide ? "  " : " "
+      r = String.new
+      height.times do |y|
+        width.times do |x|
+          r << spc.background(self[x,y].to_ansi)
+        end
+        r << "\n"
+      end
+      r
+    end
+
     def extract_block x,y=nil,w=nil,h=nil
       if x.is_a?(Hash)
         Block.new(self,x[:x], x[:y], x[:width], x[:height])
@@ -487,11 +525,10 @@ module ZPNG
     end
 
     # returns new image
-    def crop params
-      decode_all_scanlines
-      # deep copy first, then crop!
-      deep_copy.crop!(params)
+    def cropped **args
+      dup.crop!(**args)
     end
+    alias crop cropped
 
     def pixels
       Pixels.new(self)
@@ -547,5 +584,133 @@ module ZPNG
 
       new_img
     end
+
+    def _normalize_rotate x
+      x = x.to_i
+      x %= 360
+      x += 360 if x < 0
+      raise "invalid rotate: #{x}" if x%90 != 0
+      x
+    end
+
+    # always returns a copy
+    def rotated degrees
+      degrees = _normalize_rotate(degrees)
+      return dup if degrees == 0
+
+      dst = self
+      while degrees > 0
+        dst = dst.rotated_90_cw
+        degrees -= 90
+      end
+      dst
+    end
+
+    # returns new image rotated 90 degrees clockwise
+    def rotated_90_cw
+      dst = Image.new(width: height, height: width, bpp: bpp)
+      each_pixel do |c,x,y|
+        dst[height-y-1,x] = c
+      end
+      dst
+    end
+
+    def copy_from(src, copy_transparent: false,
+                  src_x: 0, src_y: 0, src_width: src.width, src_height: src.height,
+                  dst_x: 0, dst_y: 0, dst_width: src_width, dst_height: src_height)
+
+      dst_height.times do |iy|
+        dy = dst_y + iy
+        next if dy >= height
+        sy = src_y + iy * src_height / dst_height
+        next if sy >= src.height
+        dst_width.times do |ix|
+          dx = dst_x + ix
+          next if dx >= width
+          sx = src_x + ix * src_width / dst_width
+          next if sx >= src.width
+          c = src[sx,sy]
+          next if c.transparent? && !copy_transparent
+          self[dx,dy] = c
+        end
+      end
+      self
+    end
+
+    # op is a symbol of operation, like :+, :-, :* ...
+    def op_from(src, op,
+                  src_x: 0, src_y: 0, src_width: src.width, src_height: src.height,
+                  dst_x: 0, dst_y: 0, dst_width: src_width, dst_height: src_height)
+
+      dst_height.times do |iy|
+        dy = dst_y + iy
+        next if dy >= height
+        sy = src_y + iy * src_height / dst_height
+        next if sy >= src.height
+        dst_width.times do |ix|
+          dx = dst_x + ix
+          next if dx >= width
+          sx = src_x + ix * src_width / dst_width
+          next if sx >= src.width
+          c = src[sx,sy]
+          self[dx,dy] = self[dx,dy].send(op, c)
+        end
+      end
+      self
+    end
+
+    def scale(x, y=x)
+      dst = Image.new(width: width*x, height: height*y, bpp: bpp)
+      dst.copy_from(self, dst_width: dst.width, dst_height: dst.height)
+    end
+
+    alias scaled scale
+
+    def shear(mx, my)
+      src = self
+      dst = Image.new(width: width+(mx*height).abs, height: height+(my*width).abs, bpp: bpp)
+      xadd = mx < 0 ? src.height: 0
+      yadd = my < 0 ? src.width : 0
+      each_pixel do |c,x,y|
+        dst[xadd + (x+mx*y).to_i, yadd + (y+my*x).to_i] = c
+      end
+      dst
+    end
+
+    def empty?
+      pixels.all?(&:transparent?)
+    end
+
+    def * value; op(:*, value); end
+    def / value; op(:/, value); end
+    def + value; op(:+, value); end
+    def - value; op(:-, value); end
+
+    def op op, value
+      case value
+      when Image
+        dst = Image.new(width: width, height: height, bpp: bpp)
+        each_pixel do |c,x,y|
+          dst[x,y] = c.send(op, value[x,y])
+        end
+        dst
+      when Color
+        dst = Image.new(width: width, height: height, bpp: bpp)
+        each_pixel do |c,x,y|
+          dst[x,y] = c.send(op, value)
+        end
+        dst
+      else
+        raise ArgumentError, "cannot #{op} Image by #{value}"
+      end
+    end
+
+    def divmul c1, c2
+      dst = Image.new(width: width, height: height, bpp: bpp)
+      each_pixel do |c,x,y|
+        dst[x,y] = c.divmul(c1, c2)
+      end
+      dst
+    end
   end
 end
